class MERComplete(PWReportPage):
	def init(self):
		super(MERComplete, self).init()
		self.project				= None
		self.projectRateFormatter   = None
		self.plannedWorkFormatter   = None
		self.actualWorkFormatter    = None		
		self.withObjective			= True
		self.numberMode				= 0
		self.iterIndex				= 0
		self.deeperIterIndex        = 0
		self.levelDepth             = 10
		self.withAssignments        = False
		self.iterItem				= None
		return self
		

	def totalDays(self, item):
		totalHours = 0
		startDate = item.expectedStartDate()
		endDate = item.expectedEndDate()

		for activity in item.visibleSimpleActivities():
			totalHours = totalHours + activity.expectedWorkingUnitsBetweenStartDate_endDate_(startDate, endDate)
			
		return totalHours/float(self.project.manDayHours())

	
	def expectedCompleteDays(self, item):
		expectedHours = 0
		startDate = item.expectedStartDate()
		endDate = NSDate.systemNowInGMT()

		for activity in item.visibleSimpleActivities():
			expectedHours = expectedHours + activity.expectedWorkingUnitsBetweenStartDate_endDate_(startDate, endDate)
			
		return expectedHours/float(self.project.manDayHours())
		
	
	def expectedCompletePercent(self, item):
		expectedCompletePercent = 0
		
		expectedCompleteDays=self.expectedCompleteDays(item)
		totalDays = self.totalDays(item)
			
		if totalDays!=0:
			expectedCompletePercent =  (expectedCompleteDays / totalDays) * 100	
		else:
			startDate = item.expectedStartDate()
			if startDate <= NSDate.systemNowInGMT():
				expectedCompletePercent = 100
                	
		return expectedCompletePercent
	
	
	def actualCompleteDays(self, item):
		
		totalDays = self.totalDays(item)
		actualCompleteDays = float(item.actualCompletion()) * totalDays
			
		return actualCompleteDays
	
	
	def calculatedLateDays(self, item):
		calculatedLateDays = self.actualCompleteDays(item) - self.expectedCompleteDays(item)
			
		return calculatedLateDays
	
	
	def calculatedLatePercent(self, item):
		calculatedLatePercent = item.actualCompletion()*100 - self.expectedCompletePercent(item)
			
		return calculatedLatePercent
		
		
	def flag(self, item): 
		image = ""
		status = item.flagStatus()
		
		if status == 1:
			image = "FlagGreen.png"
		if status == 2:
			image = "FlagYellow.png"
		if status == 3:
			image = "FlagRed.png"
		return image


	def calculatedLateClass(self, item):
		calculatedLateDays = self.calculatedLateDays(item)
		
		if calculatedLateDays > 0:
			return "green"
		else:
			if calculatedLateDays == 0:
				return "neutral"
			else:
				return "red"



	def calculatedLatePercentOfActivity(self):
		return "%1.0f" % self.calculatedLatePercent(self.iterItem)
		
		

	def expectedCompletePercentOfActivity(self):
		return "%1.0f" % self.expectedCompletePercent(self.iterItem)



	def expectedStartDateOfActivity(self): 
		return self.iterItem.expectedStartDate()

		
		
	def expectedEndDateOfActivity(self): 
		return self.iterItem.expectedEndDate()
		


	def calculatedLateDaysOfActivity(self):	
		return "%1.0f" % self.calculatedLateDays(self.iterItem)
	
	
		
	def flagOfActivity(self):
		return self.flag(self.iterItem)	
	
		

	def calculatedLateClassOfActivity(self):
		return self.calculatedLateClass(self.iterItem)
		
	
	
		

	def plannedWorkFormatter(self):
		if self.plannedWorkFormatter == None:
			self.plannedWorkFormatter = MEAssignment.formatterForPropertyWithKey_project_("plannedWork",self.project)
		return self.plannedWorkFormatter

		



	def showObjective(self):
		return self.project.itemDescription()



	
	def addActivitiesAndChildActivitiesDeep(self, activities, result):
		if activities:
			for aAct in activities:
				if aAct.level() <= self.maxLevel:
					if ((self.levelDepth == 0) or (aAct.level() <= self.levelDepth)) :
						# Vicky Stamatopoulou recommends the below commented line. 
						# Marco Jorge prefers the using "totalDays" because it also excludes group activities that only have milestones (Yes, I do that...).
						# PS: Yes, performance wise is not the best solution...but this is not really a realtime system :)
						# if (self.showMs==0 and not aAct.isMilestone()) or self.showMs==1:
						totalDays = self.totalDays(aAct)
						if (self.showMs==0 and totalDays>0) or self.showMs==1:
							if self.withAssignments:
								result.append(aAct)
							else:
								if aAct.isAssignment() == 0:
									result.append(aAct)
							self.addActivitiesAndChildActivitiesDeep(aAct.visibleSubActivitiesArray(), result)
		return	



	def getActsWithGroups(self):
		result = []
		activities = self.project.visibleSubActivitiesArray()
		result.append(self.project)
		if activities:
			startDateDescriptor = NSSortDescriptor.alloc().initWithKey_ascending_("flatOrder", True)
			self.addActivitiesAndChildActivitiesDeep(activities, result)	
			return NSArray.arrayWithArray_(result).sortedArrayUsingDescriptors_([startDateDescriptor])
		else:
			return NSArray.array()
					
					
	def spaceByLevel(self):
		many = self.iterItem.level()		
		return many*15
	
	def rowClass(self):
		if self.iterIndex%2:
			return "even"

	def dateFormat(self):
		return "medium"


